/******************************************
ABSENTEE.PRG 

This is a Gauss program.
It replicates the empirical work reported in my paper
"Recounts from Undervotes: Evidence from the 2000 Presidential Election"
September, 2001
Final Revision: January, 2003

The program calls the companion file "beta.prc", which should be in the same directory.
It loads the data file "absentee.txt", and creates an output file "absentee.out".  

This program could be modified to estimate datasets using the multivariate BetaLogit likelihood.

The program uses the Gauss package "OPTMUM" and call Gauss graphics, so both libraries must be active. 
That is, issue the command
library pgraph,optmum;

Bruce Hansen
bhansen@ssc.wisc.edu
http://www.ssc.wisc.edu/~bhansen/

*****************************************/
cls;
library pgraph,optmum;
graphset;
pqgwin many;

#include beta.prc;
output file=absentee.out reset; 
"*****************************************";
"";"";
output off;

/****************Data****************************/

n=282;
load dat[n,6]=absentee.txt;

/*****************************************/
pn=2;				@ number of models @
y1=dat[.,3 4];			@ first model - dependent variables @
y2=dat[.,5 6];
y2=y2.*(y2.>0);
ballots=y1+y2;

title1="Gore Percentage in Machine-Readable Vote";
title2="Gore Percentage in Recount";

xc=ones(n,1);x=xc;
let xcnames = "Const" ;xnames=xcnames;pnames=xcnames;

let xselect1 = 0;		
let xselect2 = 0;		

let pselect1 = 0 ;		
let pselect2 = 0 ;

let ipselect1 = 0;			
let ipselect2 = 1;			

/*****************************************/
@ Estimation @

optset;  
_opgdprc=&G_Like_Beta;
_ophsprc=&H_Like_Beta;
_opalgr=5;
_opshess=1;
p=zeros(n,pn);
e=zeros(n,pn);

rt=zeros(pn,1);
j=1; do while j<=pn;
  nj = ftocv(j,1,0);
  _y=varget("y" $+ nj);
  _w=((_y[.,1]+_y[.,2]).>0);
  xselect=varget("xselect" $+ nj);
  if sumc(xselect)>0; xs=x[.,xselect]; else; xs=0; endif;
  _x=xp(xs,p,j);
  phat=sumc(_y[.,1].*_w)/sumc(sumc(_y.*_w));
  theta0=zeros(cols(_x),1)|(1/phat/4); 
  theta0[1]=ilogit(phat);
  {theta,f,gr,ret}=optmum(&Like_Beta,theta0);
  g=V_G_Like_Beta(theta,_y,_x);
  gr=sumc(g)';
  h=H_Like_Beta(theta);
  p[.,j] = pr_hat(_y,_x,theta);
  e[.,j]=(_y[.,1]./(_y[.,1]+_y[.,2]+(1-_w))-logit(_x*theta[1:cols(_x)])).*_w;
  sj = varput(phat,"phat" $+ nj);
  sj = varput(_x,"x" $+ nj);
  sj = varput(xs,"xs" $+ nj);
  sj = varput(_w,"_w" $+ nj);
  sj = varput(theta,"theta" $+ nj);
  sj = varput(f,"f" $+ nj);
  sj = varput(g,"g" $+ nj);
  sj = varput(gr,"gr" $+ nj);
  sj = varput(h,"h" $+ nj);
  rt[j]=rows(theta);
j=j+1;endo;

/*****************************************/
@ Covariance Matrix @
r=sumc(rt);
h=zeros(r,r);
g=varget("g" $+ ftocv(1,1,0));
crt=0|cumsumc(rt);
j1=1; do while j1<=pn;
  rj1=rt[j1];
  nj = ftocv(j1,1,0);
  if j1>1;    g=g~varget("g" $+ nj); endif;
  h[crt[j1]+1:crt[j1]+rj1,crt[j1]+1:crt[j1]+rj1]=varget("h" $+ nj);
  theta=varget("theta" $+ nj);
  j2=1; do while j2<=rj1;
    thetaj2=theta[j2];
    ax=abs(thetaj2);
    delta=((ax.>1).*ax + (ax.<=1)).*(1e-6);
    theta[j2]=thetaj2+delta;
    pj=p;
    xs=xp(varget("xs" $+ nj),pj,j1);
    pj[.,j1] = pr_hat(varget("y" $+ nj),xs,theta);
    j3=(j1+1); do while j3<=pn;
      nj3 = ftocv(j3,1,0);
      y=varget("y" $+ nj3);
      xs=xp(varget("xs" $+ nj3),pj,j3);
      _w=varget("_w" $+ nj3);
      thetaj3=varget("theta" $+ nj3);
      pj[.,j3] = pr_hat(y,xs,thetaj3);
      h[crt[j1]+j2,crt[j3]+1:crt[j3+1]]=(sumc(V_G_Like_Beta(thetaj3,y,xs))'-varget("gr" $+ nj3))./delta;
    j3=j3+1;endo;
    theta[j2]=thetaj2;
  j2=j2+1;endo;
j1=j1+1;endo;
v=inv(h);
v=v'moment(g,0)*v;

/*****************************************/
@ Print Results @
format 16,8;
let fmt1[1,3]="*.*lf" 14 8;
let fmt2[1,3]="*.*lf" 14 6;
let fmt3[1,3]="*.*lf" 8 3;
let fmt4[1,3]="*.*lf" 8 8;
output on;
rr=0;
j=1; do while j<=pn;
  nj = ftocv(j,1,0);
  tit=varget("title" $+ nj);
  y=varget("y" $+ nj);
  _w=varget("_w" $+ nj);
  theta=varget("theta" $+ nj);
  rj=rows(theta);
  ij=seqa(rr+1,1,rj);
  vj=v[ij,ij];
  sj = varput(vj,"v" $+ nj);
  rr=rr+rj;
  xselect=varget("xselect" $+ nj);
  pselect=varget("pselect" $+ nj);
  ipselect=varget("ipselect" $+ nj);
  names=xcnames;
  if sumc(xselect)>0;
    names=names|xnames[xselect];
  endif;
  if sumc(pselect)>0;
    names=names|pnames[pselect];
  endif;
  if ipselect==1;
    names=names|xcnames;
  endif;
  s=theta[rj];
  phat=varget("phat" $+ nj);
  sig=sqrt(phat.*(1-phat)./(1+s));
  th=theta|sig;
  se=sqrt(diag(vj));
  se=se|(se[rj]*sig/(2*(1+s))); 
  $tit;
  "Number of Precincts " sumc(_w);
  "Totals              " sumc(y)';
  "Percentage          " phat;
  "";
  pr=printfm("Variable"~"Estimate"~"St Error",0~0~0,fmt1);"";
  pr=printfm((names|"Scale"|"SD")~th~se,0~1~1,fmt2);  "";
  "Negative Log Likelihood " varget("f" $+ nj);
  "";"";
  "*****************************************";
j=j+1;endo;
output off;

/*****************************************/
@  Plot Graphs @
npoints=199;
xpoints=seqa(1/(npoints+1),1/(npoints+1),npoints);

graphset;
_pcolor=15;
_pdate="";
_plwidth=3;


j1=1; do while j1<=pn;
  nj1 = ftocv(j1,1,0);
  ipselect=varget("ipselect" $+ nj1);
  if sumc(ipselect) /= 0;
    pselect=varget("pselect" $+ nj1);
    theta=varget("theta" $+ nj1);
    _w=varget("_w" $+ nj1);
    xselect=varget("xselect" $+ nj1);
    p1=ilogit(xpoints);
    if pselect /= 0;
      p2=sumc(ln(p[.,pselect]).*_w.*ballots)./sumc(_w.*ballots);
    endif;  
    v=varget("v" $+ nj1);
    v=v[1:rows(v)-1,1:rows(v)-1];
    tt= 1;
    tn=1; 
    z1=ones(npoints,1);
    if sumc(xselect)>0; 
      cx=rows(xselect);
      tt = tt|(seqa(tn+1,1,cx));
      tn=tn+cx;
      z1=z1~ones(npoints,cx);
    endif;
    if sumc(pselect)>0; 
      tt = tt|(seqa(tn+1,1,rows(pselect)));
      tn=tn+rows(pselect);
      z1=z1~(ones(npoints,1)*(p2'));
    endif;
    tt=tt|(tn+1);
    theta1=theta[tt[.,1]];
    v1=v[tt[.,1],tt[.,1]];
    z1=z1~p1;
    zt=z1';
    mu1=logit(z1*theta1);
    se1=sqrt(mu1.*(1-mu1).*sumc((v1*zt).*zt));
    a1=mu1-se1*1.96;    b1=mu1+se1*1.96;
    let _pltype = 6 3 3 4;
    xlabel("p");
    ylabel("E(q|p)");
   
    ytics(0,1,.1,10);
    tit="" $+ "Figure 4\L" $+ varget("title" $+ nj1) $+ "\LAbsentee Precincts";
    title(tit);
    xy(xpoints,mu1~a1~b1~xpoints);
  endif;
j1=j1+1;endo;

/*****************************************************/
@ Procedures @

proc xp(xs,p,j);		@ generates covariate X matrix @
local pselect,ipselect,xx,xselect;
  pselect=varget("pselect" $+ ftocv(j,1,0));
  xselect=varget("xselect" $+ ftocv(j,1,0));
  ipselect=varget("ipselect" $+ ftocv(j,1,0));
  xx=xc;
  if sumc(xselect)>0;
    xx=xx~xs;
  endif;
  if sumc(pselect)>0;
    xx=xx~ln(p[.,pselect]); 
  endif;
  if ipselect==1;
    xx=xx~(ilogit(p[.,1]).*xc); 
  endif;
retp(xx);
endp;

fn pr_hat(y,x,theta)=(logit(x*theta[1:cols(x)]).*theta[rows(theta)]+y[.,1])./(y[.,1]+y[.,2]+theta[rows(theta)]);
fn expt(x)=exp(x.*(x.>(-700)).*(x.<700)-(x.<=(-700))*700+(x.>700)*700);
fn logit(u)=1./(1+expt(-u));
fn ilogit(y)=-ln((1./y)-1);